/**
 * Copyright (c) 2010 jolira. All rights reserved. This program and the accompanying materials are made available under
 * the terms of the GNU Public License 2.0 which is available at http://www.gnu.org/licenses/old-licenses/gpl-2.0.html
 */

package com.google.code.joliratools.logdb.parser;

import java.io.IOException;
import java.io.InputStream;

/**
 * A parser for processing delimited files.
 * 
 * @author jfk
 * @date Sep 13, 2010 11:09:15 PM
 * @since 1.0
 */
public class DelimitedParserFactory implements ParserFactory {
    private static int max(final int... vals) {
        int max = vals[0];

        for (int idx = 1; idx < vals.length; idx++) {
            if (vals[idx] > max) {
                max = vals[idx];
            }
        }

        return max;
    }

    private final String fieldDelimiter;
    private final int timeStampPos;
    private final String timeStampFormat;
    private final int maxPos;
    private final int classPos;
    private final int methodPos;
    private final int levelPos;

    /**
     * @param fieldDelimiter
     *            the delimiters to be used to break the records into fields
     * @param timeStampPos
     *            the position of the timestamp field
     * @param timeStampFormat
     *            the format of the timestamp field
     * @param classPos
     *            the position of the class name
     * @param methodPos
     *            the position of the method name
     * @param levelPos
     *            the position of the field containing the level
     */
    public DelimitedParserFactory(final String fieldDelimiter, final int timeStampPos, final String timeStampFormat,
            final int classPos, final int methodPos, final int levelPos) {
        this.fieldDelimiter = fieldDelimiter;
        this.timeStampFormat = timeStampFormat;
        this.timeStampPos = timeStampPos;
        this.classPos = classPos;
        this.methodPos = methodPos;
        this.levelPos = levelPos;

        maxPos = max(timeStampPos, classPos, methodPos, levelPos);
    }

    @Override
    public Parser parse(final InputStream in) throws IOException, ParseException {
        return new DelimitedParser(in, fieldDelimiter, timeStampFormat, timeStampPos, classPos, methodPos, levelPos,
                maxPos);
    }

}
