/**
 * Copyright (c) 2010 jolira. All rights reserved. This program and the accompanying materials are made available under
 * the terms of the GNU Public License 2.0 which is available at http://www.gnu.org/licenses/old-licenses/gpl-2.0.html
 */
package com.google.code.joliratools.logdb.reader;

import java.io.IOException;
import java.text.ParseException;
import java.util.Map;

import com.google.code.joliratools.logdb.parser.Parser;
import com.google.code.joliratools.logdb.parser.Record;
import com.google.code.joliratools.logdb.parser.Record.Type;

/**
 * @author jfk
 * @date Sep 16, 2010 7:41:31 AM
 * @since 1.0
 */
public abstract class LogDigester implements Reader {
    /**
     * @see Reader#getNext(Parser)
     */
    @Override
    public Entry getNext(final Parser parser) throws IOException, ParseException {
        final Record record = parser.getNext();

        if (record == null) {
            return null;
        }

        return new Entry() {
            @Override
            public String getClazz() {
                return record.getClazz();
            }

            @Override
            public String getId() {
                return record.getId();
            }

            @Override
            public String getLevel() {
                return record.getLevel();
            }

            @Override
            public int getLineNumber() {
                return record.getLineNumber();
            }

            @Override
            public String getMethod() {
                return record.getMethod();
            }

            @Override
            public String getRaw() {
                return record.getRaw();
            }

            @Override
            public long getTimestamp() {
                return record.getTimestamp();
            }

            @Override
            public Type getType() {
                return record.getType();
            }

            @Override
            public Map<String, String> getVariables() {
                return LogDigester.this.getVariables(record);
            }
        };
    }

    /**
     * @param record
     *            the record
     * @return the process variables, or {@literal null} if the record is junk or unrecognized.
     */
    protected final Map<String, String> getVariables(final Record record) {
        final Type type = record.getType();

        if (!Type.RECOGNIZED.equals(type)) {
            return null;
        }

        return processVariables(record);
    }

    /**
     * @param record
     *            the record to be processed
     * @return the variables by name.
     */
    protected abstract Map<String, String> processVariables(Record record);
}
